/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * Probe.java
 *
 * Created on May 4, 2003, 7:20 AM
 */

package com.securitycentric.metacoretex.interfaces;

import com.securitycentric.metacoretex.lib.*;

import java.util.*;

/** This class defines the interface which must be implemented by all probes
 * @author visigoth
 * @version $Id: Probe.java,v 1.7 2003/10/02 02:20:44 visigoth_6 Exp $
 */
public interface Probe extends Runnable {
    
    /** Each probe will have a unique ID assigned.  Get it.
     * @return int probeId
     */
    public int getProbeId();
    
    /** Set the probeId of the probe.  Each probe should call this upon initialization
     * @param x int which will be used to uniquely identify each probe
     */    
    public void setProbeId(int x);
    
    /** The getFamily() method should return a string representing what
     *  interface this probe implements.  Most probes can just rely on their underlying
     * abstract probe to report this correctly.
     * @return String representing the family of probes this probe
     * belongs to.
     */
    public String getFamily();
    
    /** Use the setFamily() method to set the family (more importantly the
     *  tab in the UI) for this probe.  Family names are used for ONLY the
     *  GUI rendering..
     *  @param newFam String name of the new probe family
     */
    public void setFamily(String newFam);

    /** The getName() method should return a short unique name for the
     *  probe to use in descriptions.
     * @return String with a short descriptive name for this probe
     */
    public String getName();
    
    /** Set the name of this probe.  This should likely be a
     * setter for a string held in the probe implemntation.
     * @param myname String representation of the name of this probe
     *
     */
    public void setName(String myname);
    
    /** This function should return true ONLY if the function probeSafe()
     *  is capable of detecting the specified vuln.
     * @return boolean which is true if this probe's probeSafe() method
     * is capable of detecting the issue
     */
    public boolean hasSafe();
    
    /** Carry out the probe.  Throw a ProbeException only if test cannot
     *  determine (possibly due to network failure or other).
     *
     * @throws com.securitycentric.metacoretex.lib.ProbeException These exceptions are thrown when failure causes the probe to be unable to determine weather or not the target is vulnerable.
     */
    public void probe() throws ProbeException;
    
    /** Carry out the _safe_ version of the probe.  These activities should
     *  not be likely to hurt the target.  If whole test is safe, probeSafe()
     *  is encouraged to call probe()
     *  @throws com.securitycentric.metacoretex.lib.ProbeException These exceptions are thrown when failure causes the probe to be unable to determine weather or not the target is vulnerable.
     */
    public void probeSafe() throws ProbeException;
    
    /** Give the probe it's target
     * @param targ set the com.securitycentric.metacoretex.lib.Target
     * object this probe will use to carry out attacks
     */
    public void setTarget(Target targ);
    
    /** Return this probe's target
     * @return Return a reference to this probe's current Target
     *
     */
    public Target getTarget();
    
    /** Checks to see if the current probe is enabled.
     * @return Return true if this probe is enabled, false if not.
     */
    public boolean isEnabled();
    
    /** Sets the boolean enabled status of this probe
     * @param bol Set the enabled status of this probe
     */
    public void setEnabled(boolean bol);

    /** Return a vector of the KB keys this probe
     *  requires to run.
     * @return Vector of the KB keys this probe depends upon
     */
    public Vector depends();

    /** Return a Hashtable of the KB keys this probe
     *  provides sets and their descriptions.  This is mostly used by the dependency
     * resolver which orders the probe for firing
     * @return Hashtable of the KB's this probe provides
     */
    public Hashtable provides();
    
    /** This function returns a Vector of the keys in the Properties
     * table which represents this probe's configuration options
     * @return Vector of the keys (names) of the configuration options
     * for this probe.
     */    
    public Vector getOptionKeys();
    
    /** This method provides the probe handling subsystems with
     * a way to get a reference to the Properties objec which stores
     * the configuration options for this probe.
     * @return Properties object of the configuration options for this probe
     *
     */    
    public Properties dumpOptions();
    
    /** This method is used by the scan configuration loader to
     * override the default configuration options for the probe.
     * @param prop New Properties object for this probe to use.
     */    
    public void replaceOptions(Properties prop);
    
    /** This method adds an element to the underlying implementations
     * list of depends for this probe.  Used by the dep. resolver
     * later.  Each probe should call this method to specify any
     * KB resources upon which they depend.
     * @param name String representation of the KB key for the value this
     * probe depends upon.
     */    
    public void addDepends(String name);
    
    /** This method adds an element to the underlying implementations
     * list of resources this probe provides.  Used by the dep. resolver
     * later.  Each probe should call this method to specify any
     * KB resources which they provide.
     * @param name String representation of the KB key which this
     * @param descr String description of the provided KB key
     * probe provides.
     */    
    public void addProvides(String name, String descr);
    
    /* this is for backward compatability and will be removed soon
     */
    //public void addProvides(String name);
    
    /** This method returns a String which represents the current
     * reportable information in the probe.  Used later by
     * Attacker threads to fill in ProbeReport objects.
     * @return String of what the probe has to report
     */    
    public String getReport();

    /** This method sets (overwrites) the current report String value.  Abstract
     * implementations should take care of this automagically.
     * @param rep String of report data
     */    
    public void setReport(String rep);
    
    /** This method concats the specified string to the end of the
     *  existing report string and appends a newline.  Makes it easier
     *  for probes which often are adding single lines of info.
     *  @param rep String string to be concated to current report string
     */
    public void addReport(String rep);
    
    /** This method adds a key->value pair to the options table for this
     * probe.  Any probes which add options upon instantiation will be
     * able to receive user config options by their use later.
     * @param name String representation of the Properties key (name)
     * for this option
     * @param value String representation of the value for this configuration option.
     */    
    public void addOption(String name, String value);
    
    /** This method returns the value of the specified option key.
     * @param name String representation of the key for the key->value
     * pair.
     * @return String value of the specified configuration option.
     */    
    public String getOption(String name);
    
    /** Set the version string for this probe.
     * @param ver String version
     */    
    public void setVersion(String ver);
    
    /** Get the version string for this probe.
     * @return String version
     */    
    public String getVersion();
    
    /** The kbGet() method wraps the kbGet() method for the currently
     * set target.  This allows probes to access the current Target KB
     * regardless of the implementation.  Probes should call this method
     * to retreive any Objects from the KB upon which they depend.
     * @param key String representation of the key (name) of the
     * option in the Target KB
     * @return Object which is the value of the Target KB key
     */    
    public Object kbGet(String key);
    
     /** The kbPut() method wraps the underlying Target kbPut() method
      * to allow probes access to Target KB's regardless of implementaion.
      * Probes should use this method to set any Objects into the
      * Target KB which they declaired they provide (addProvides())
      * @param key String key for the Target KB
      * @param obj Object to insert into the KB
      */    
    public void kbPut(String key, Object obj);
    
    /** This method wraps the underlying Target's kbHas() method to
     * determine if the current Target has the specified key in it's
     * current KB.
     * @return boolean specifying weather or not the specified key
     * is in the KB
     * @param key String key to test for in the KB
     */    
    public boolean kbHas(String key);
    
    /** The getHost() method wraps the current Target's getHost()
     * method to allow the probe to know who it's supposed to poke.
     * @return String of the hostname/IP of the current Target
     *
     */    
    public String getHost();
    
    /** This method returns the severity (0-3) of the report the
     * probe would like to make.
     * @return int value represents the severity (0-3) where:
     * 0 - Informational
     * 1 - Low Risk
     * 2 - Medium Risk
     * 3 - High Risk
     */    
    public int getSeverity();
    
    public boolean isReportable();
    
    /** Set the probe to be reportable, which makes the attacker threads
     *  add it's report to the target upon probe completion
     */
    public void setReportable(boolean x);
    
    public ProbeException getException();
    
    /** This method sets the severity of the report which the probe
     * will return to the attacker threads.
     * @param x int representation of the severity (see getSeverity() for definitions)
     */    
    public void setSeverity(int x);
    
    /** Set the String value returened by the getHelp() method (used by the UI
     *
     * @param help String help/config information
     */
    public void setHelp(String help);
    
    /** Return the Help String for this probe
     * @return String help string
     */
    public String getHelp();
    
    /** Set the copyright string for this probe which is reported
     *  in the Help->about probes.
     *
     * @param String copyright string
     */
    public void setCopyright(String crght);
    
    /** Get the copyright string for the current probe.
     *
     * @return String copyright string
     */
    public String getCopyright();
    
    /** Get the timeout in milliseconds that this probe should
     *  use when running.
     */
    public int getTimeout();
    
    /** Set the timeout in milliseconds that this probe should
     *  use when running.  Leave this value alone unless you know
     *  what you are doing!
     */
    public void setTimeout(int timeout);
    
}
