/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * AbstractProbe.java
 *
 * Created on May 5, 2003, 7:26 AM
 */

package com.securitycentric.metacoretex.lib;

import com.securitycentric.metacoretex.*;
import com.securitycentric.metacoretex.lib.*;
import com.securitycentric.metacoretex.interfaces.*;

import java.sql.*;
import java.util.*;

/** This is the base abstract probe from which all probes are extended.  This implementation
 * takes care of almost all of the necissary data management functions so developers
 * can focus on the probe code alone.  To extend this probe, you may do as little as implement
 * the probe() method, but be sure to set up all the necissary information when your
 * class is instanciated!
 * @author visigoth
 */
public abstract class AbstractProbe implements Probe {
    
    /** implement the runnable interface for threading junk
     */    
    public void run() {
        // Should test for safeProbe and junk here
        //FIXME
        Thread.yield();
        try {
            if (Init.safeChecks)
                probeSafe();
            else
                probe();
        } catch (ProbeException pe) {
            pException = pe;
        }
    }
    
    /** Get the ID of this probe.
     * @return int the probe's ID
     */    
    public int getProbeId() {
        return(probeId);
    }
    
    /** Set the ID of this probe.  Probe ID's should be unique, as the loader will
     * overwrite instances of probes with the same IDs.
     * @param x int ID to set for this probe.
     */    
    public void setProbeId(int x) {
        probeId = x;
    }
    
    /** Get the String name of the family of probes to which this probe belongs.
     * @return String family name
     */    
    public String getFamily() {
        return(familyStr);
    }
     
    public void setFamily(String newFam) {
        familyStr = newFam;
    }
    
    /** Get the human readable name for this probe
     * @return String human readable name for this probe
     */    
    public String getName() {
        return(nameStr);
    }
    
    /** REQUIRED: Set the human readable name for this probe.
     * @param myname String name for this probe.
     */    
    public void setName(String myname) {
        nameStr = myname;
    }
    
    /** Find out if this probe has a "safe" attack or not
     * @return boolean true if this probe has a "safe" probe option, false if not.
     */    
    public boolean hasSafe() {
        return(safe);
    }
    
    /** Get the String of the report for the probe.
     * @return String the current reportable value for this probe.
     */    
    public String getReport() {
        return(repStr);
    }

    /** Set the report String for this probe.  This function will throw out all previously
     * set information for this report string.
     * @param newRpt String The new reportable value for this probe.
     */    
    public void setReport(String newRpt) {
        repStr = newRpt;
    }
    
    /** Add the specified string and a newline to the report.
     * @param reptAddStr String to add to the report
     */    
    public void addReport(String reptAddStr) {
        repStr = repStr.concat(reptAddStr+"\n");
    }
    
    /** Get a reference to the Target object for this probe.
     * @return Target this probe is assigned to.
     */    
    public Target getTarget() {
        return(target);
    }
    
    /** Set the Target this probe will attack on invocation of the probe() method.
     * @param targ com.securitycentric.metacoretex.lib.Target This probe's target.
     */    
    public void setTarget(Target targ) {
        this.target = targ;
    }
    
    /** Checks to see if this probe is currently enabled (will fire on launch)
     * @return boolean true if enabled. false if not
     */    
    public boolean isEnabled() {
        return(enabled);
    }
    
    /** Set the enabled flag on this probe.
     * @param bol boolean true for enabled, false for not
     */    
    public void setEnabled(boolean bol) {
        enabled = bol;
    }

    /** This method represents the "safe" option for the probe.  For most probes, this
     * method will just call probe() (if they are naturally safe probes)
     * @throws ProbeException This exception is thrown if the probe is unable to process.
     * ex. network outage. SQL failure etc.
     */    
    public void probeSafe() throws ProbeException {
        probe();
    }
    
    public Vector depends() {
        return(depVect);
    }

    public Hashtable provides() {
        return(provHash);
    }
    
    /** Return a vector of all the option KEYS
     */
    public Vector getOptionKeys() {
        Enumeration en = options.keys();
        Vector vect = new Vector();
        
        while (en.hasMoreElements())
            vect.add(en.nextElement());
            
        return(vect);
    }
    /*
    public void addProvides(String name) {
        provHash.put(name,"No Description");
    }
    */
    /** Add a KB key to the list of keys provided by this probe.
     * @param name String Name for the KB key provided by this probe
     * @param descr String Description of the Object (include type) which this probe puts
     * in the KB.
     */    
    public void addProvides(String name, String descr) {
        provHash.put(name,descr);
    }
    
    /** Add the key to the list of KB objects which this probe uses (KB objects provided
     * by other probes).
     * @param name String The KB key which is used by this probe.
     */    
    public void addDepends(String name) {
        depVect.add(name);
    }
    
    /** Add a user-configurable option with a default value.
     * @param opt String Name of the option
     * @param value String Default value for this option
     */    
    public void addOption(String opt, String value) {
        options.setProperty(opt,value);
    }
    
    /** Get the String representing the user configured option.  Usually called by
     * a probe at probe-time to get user input.
     * @param opt String The option name specified in an earlier call to addOption()
     * @return String The value specified by the user before launch
     */    
    public String getOption(String opt) {
        return(options.getProperty(opt));
    }
    
    /** Get a String representation of the version of this probe
     * @return String The version of this probe.
     */    
    public String getVersion() {
        if (version == null)
            return(unspec);
        
        return(version.toString());
    }
    
    /** Set the version string for this probe
     * @param ver String A version string like: "1.34.3_5" etc.
     */    
    public void setVersion(String ver) {
        try {
            version = new Version(ver);
        } catch (VersionException e) {
            version = null;
        }
    }
    
    public Properties dumpOptions() {
        return(options);
    }
    
    public void replaceOptions(Properties prop) {
        options = prop;
    }
    
    /** Put an Object into the Target's KB for later use by other probes.
     * @param key String The key for the KB object.  Usually this is the same as the key specified
     * in a call to addProvides() during instantiation.
     * @param obj Object The Object in insert into the KB.  Can be anything you want.
     */    
    public void kbPut(String key, Object obj) {
        target.kbPut(key,obj);
    }
    
    /** Get the Object referenced at the specified KB key
     * @param key String The key which identifies the Object in the KB
     * @return Object The Object at that key in the KB
     */    
    public Object kbGet(String key) {
        return(target.kbGet(key));
    }
    
    /** Test to see if the KB contains the specified key.  Usefull for checking if your
     * probe's dependancies have been met, and avoiding calls to check if your kbGet()
     * call returned null.
     * @param key String The key to test for in the KB
     * @return boolean true if the KB has the key, false if not
     */    
    public boolean kbHas(String key) {
        return(target.kbHas(key));
    }
    
    /** Get the name/ip for the current Target as a String.  Used by probe's to get
     * the target host for poking.
     * @return String The name or string IP of the Target this probe is assigned.
     */    
    public String getHost() {
        return(target.getHost());
    }
    
    /** Get the integer value for the severity of this probe:
     * 0 - Data Gathering - Could be good or bad, Requires analysis
     * 1 - Low Risk
     * 2 - Medium Risk
     * 3 - High Risk
     * @return int The sevrity of this probe.
     */    
    public int getSeverity() {
        return(severity);
    }
    
    /** Set the severity rating for this probe's report.  This may be called during the
     * probe() method, to set the severity based on results received.
     * @param x int The severity
     */    
    public void setSeverity(int x) {
        severity = x;
    }
    
    /** Test weather or not this probe has anything to report.
     * @return boolean Does this probe have a report?
     */    
    public boolean isReportable() {
        return(reportable);
    }
    
    /** Set the reportable status of this report.  Probes should call this with a true
     * value once they are reasonably certain they have information to report.
     * @param x boolean
     */    
    public void setReportable(boolean x) {
        reportable = x;
    }
    
    /** Return a copy of a ProbeException which may have occured during the course of
     * the probe() method.  Used by the attacker threads, and likely internal.
     */    
    public ProbeException getException() {
        return(pException);
    }
    
    /** Get a reference to the help string for the current probe.
     * @return String The help string for this probe.
     */    
    public String getHelp() {
        return(helpStr);
    }
    
    /** Set the copyright information for this probe.  Used to display information
     * in Help->about probes.
     * @param copyright String The copyright info like : (c) 2003, visigoth blah blah blah
     */    
    public void setCopyright(String copyright) {
        this.copyright = copyright;
    }
    
    /** Get the copyright string for this probe.
     */    
    public String getCopyright() {
        return(copyright);
    }
    
    /** Set the String value returened by the getHelp() method (used by the UI)
     *
     * @param hlp String help/config information
     */
    public void setHelp(String help) {
        helpStr = help;
    }
   
    public int getTimeout() {
        return(timeout);
    }    
    
    public void setTimeout(int timeout) {
        this.timeout = timeout;
    }
    
    /** this probe's ID
     */    
    public int probeId = -1;
    /** this probes current severity
     */    
    public int severity = 0;
    
    /** this probes timeout in miliseconds (setting to 42666 uses global default)
     */
    public int timeout = 42666;
    
    /** The Properties object which holds user configurable options.
     */    
    public Properties options = new Properties();

    /** The target which this probe will poke.
     */    
    public Target target;
    /** an exception to throw if things go bad.
     */    
    public ProbeException pException = null;
    /** The Report string
     */    
    public String repStr = "No Report";
    /** the human readable name of this probe
     */    
    public String nameStr = "Probe X";
    
    /** the Version object for the version of this probe
     */    
    public Version version = null;
    /** a string to use for junk that developers didn't initialize.
     */    
    public String unspec = "unknown";
    /** the copyright string for this probe
     */    
    public String copyright = "unspecified";
    /** the help String for this probe
     */    
    public String helpStr = "No Help Available";

    /** probes may be disabled by user input, store it in the probe.
     */    
    public boolean enabled = true;
    /** keep track of weather or not this probe is safe.
     */    
    public boolean safe = true;
    /** does this probe have anything to report?
     */    
    public boolean reportable = false;

    /** Vector of KB keys (Strings) this probe depends on
     */    
    public Vector depVect = new Vector();
    /** key->value pairs of kbentry->description
     */    
    public Hashtable provHash = new Hashtable();
    
    /** the name of the family for this probe which is used in GUI rendering
     */
    public String familyStr = "Misc";
}
