/*
 *(c) 2003, visigoth <visigoth@securitycentric.com>
 * This source code is distributed under a BSD license,
 * please see the LICENSE file for details
 *
 * AbstractVersionProbe.java
 *
 * Created on May 29, 2003, 4:40 AM
 *
 * $Id: AbstractVersionProbe.java,v 1.5 2003/09/21 22:42:13 visigoth_6 Exp $
 *
 */

package com.securitycentric.metacoretex.lib;

/** One of the other Abstract probe classes which doesn't require the developer
 * to implement a probe() method.  These probes usually just test their specified
 * version against a KB key set by another probe which gathered verion information
 * from a Target.
 * @author visigoth
 */
public class AbstractVersionProbe extends AbstractProbe implements com.securitycentric.metacoretex.interfaces.VersionProbe {
    
    /** Get the KB key used for the Version object this probe tests.
     * @return String The KB key for the Version object this probe comapres to.
     */    
    public String getKbKey() {
        return(kbKey);
    }
    
    /** Get the Version object which this probe has set for itself.  This is used to
     * compare to a KB key during the AbstactVersionProbe probe() method.
     * @return com.securitycentric.metacoretex.lib.Version to test
     */    
    public Version getTestVersion() {
        return(testVer);
    }
    
    /** Determines if this VersionProbe must match the release to bother testing
     * @return boolean true if this Version must match release in order to test.
     */    
    public boolean matchRelease() {
        return(match);
    }
    
    /** Set the KB key which is used by this probe to get the Version to test agains.
     */    
    public void setKbKey(String key) {
        kbKey = key;
    }
    
    /** Set the value for the match release parameter.  If true, this probe will only
     * test versions if the release version (like the 4 in 4.23.5) matches.
     */    
    public void setMatchRelease(boolean mat) {
        match = mat;
    }
    
    /** Set the version string tested by this probe. This should be the
     *  value of the first FIXED version of the application in question.
     *  For example, if MySQL is vulnerable up to 3.23.55, you would test for
     *  3.23.56...
     */    
    public void setTestVersion(String ver) {
        try {
            testVer = new Version(ver);
        } catch (VersionException ve) {
            testVer = null;
        }
    }
    
    /** Carry out the probe.  Throw a ProbeException only if test cannot
     *  determine (possibly due to network failure or other).
     *
     * @throws com.securitycentric.metacoretex.lib.ProbeException These exceptions are thrown when failure causes the probe to be unable to determine weather or not the target is vulnerable.
     */
    public void probe() throws ProbeException {
        
        //System.out.println("running "+getName());
        if (testVer == null) 
            throw(new ProbeException("Must set test version to ver string like 3.3.5232.3"));
        if (kbKey.matches("none"))
            throw(new ProbeException("Must set KB key (like oracle/Version)"));
        
        // If we didn't find our key, silently return
        if (!kbHas(kbKey))
            return;
        
        Object obj = kbGet(kbKey);
        
        if (! (obj instanceof Version))
            throw(new ProbeException("object at "+kbKey+" is a "+obj.getClass().getName()+" and should be Version"));
        
        Version targVer = (Version)obj;
        
        // Test for needing to match release version to test
        if (match) {
            if (targVer.verInt(0) != testVer.verInt(0))
                return;
        }
        
        if ( targVer.isOlderThan(testVer))
            setReportable(true);
    }
    
    String kbKey = "none";
    boolean match = false;
    Version testVer;
}
